"""Top-level package for fsds."""

__author__ = """James Irving"""
__email__ = 'james.irving.phd@gmail.com'
__version__ = '0.2.26'

from .default import *

from fsds import datasets,jmi,quick_refs


# from fsds_100719 import datasets
# from fsds_100719 import quick_refs
# from .fsds import imports 